// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.services' is found in services.js
// 'starter.controllers' is found in controllers.js
angular.module('starter', ['ionic', 'starter.controllers', 'starter.services', 'starter.directives', 'parse-angular'])

.run(function($ionicPlatform, $state, $rootScope, $location, Auth) {
  $ionicPlatform.ready(function() {
    // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
    // for form inputs)
    if (window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard) {
      cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
    }
    if (window.StatusBar) {
      // org.apache.cordova.statusbar required
      StatusBar.styleLightContent();
    }

    var loginRequiredPaths = [];
    var additionalPaths = ['/reply/'];

    angular.forEach($state.get(), function(route) {
        if(route.loginRequired) {
            loginRequiredPaths.push(route.url);
        }
    });

    $rootScope.$on('$stateChangeStart', function(event, nextLoc, currentLoc) {
        if  (
            (loginRequiredPaths.indexOf($location.path())!=-1 && !Auth.isUserLoggedIn()) || ($location.path().search('/reply/') != -1 && !Auth.isUserLoggedIn())
            )
        {
            $location.path('/login');
        }
    });

  });
})

.config(function($stateProvider, $urlRouterProvider) {

  // Ionic uses AngularUI Router which uses the concept of states
  // Learn more here: https://github.com/angular-ui/ui-router
  // Set up the various states which the app can be in.
  // Each state's controller can be found in controllers.js
  $stateProvider

    // setup an abstract state for the tabs directive
    .state('posts', {
    url: "/posts",
    abstract: true,
    templateUrl: "templates/home.html",
    controller: function($scope){
            $scope.post_type = "new";
            $scope.update_post_type = function(new_post_type) {
                $scope.post_type = new_post_type;
            };
        }
    })

  // Each tab has its own nav history stack:

    .state('posts.type', {
        url: '/:post_type',
        views: {
            'content': {
                templateUrl: 'templates/home-posts.html',
                controller: 'PostsCtrl'
            }
        },
        cache: false
    })

    .state('createPost', {
        url: '/post/create',
        templateUrl: 'templates/post-create.html',
        controller: 'CreatePostCtrl',
        loginRequired: true,
        cache: false
    })

    .state('createComment', {
      url: '/reply/:post_id',
      templateUrl: 'templates/comment-create.html',
      controller: 'CreateCommentCtrl',
      loginRequired: true
    })

    .state('comments', {
        url: '/post/:post_id',
        templateUrl: 'templates/post-detail.html',
        controller: 'PostCtrl',
        cache: false
    })



    //
    //.state('home.reply', {
    //    url: '/reply/:postId',
    //    views: {
    //    'tab-chats': {
    //      templateUrl: 'templates/post-reply.html',
    //      controller: 'ReplyCtrl'
    //    }
    //    }
    //})
    //
    .state('login', {
        url: '/login',
        templateUrl: 'templates/login.html',
        controller: 'LoginCtrl'
    })

    .state('register', {
      url: '/register',
      templateUrl: 'templates/register.html',
      controller: 'RegisterCtrl'
    })


  // if none of the above states are matched, use this as the fallback
  $urlRouterProvider.otherwise('/posts/new');

});
