angular.module('starter.controllers', [])

.controller('PostsCtrl', function($scope, $stateParams, Utility, $ionicPopup, $ionicScrollDelegate, $state, $location, Posts, Auth) {
    $scope.post_type = $stateParams.post_type;
    $scope.$parent.update_post_type($scope.post_type);
    $scope.loading = true;
    $scope.posts = undefined;

    /** get the current users likes first if any **/
    var PromiseChain = undefined;
    var retrievePostsPromise = undefined;

    Posts.refresh();

    if ($scope.post_type=='trending') {
        retrievePostsPromise = Posts.trendingPosts();
    } else if ($scope.post_type=='popular') {
        retrievePostsPromise = Posts.topPosts();
    } else {
        retrievePostsPromise = Posts.newPosts();
    }

    if (Auth.isUserLoggedIn()) {
        PromiseChain = Auth.retrieveParseUsersLikes().then(function() {
            return retrievePostsPromise.fetch();
        })
    } else {
        PromiseChain = retrievePostsPromise.fetch();
    }
    //, function(reason) {
    //    $ionicPopup.show({
    //        template: ''+reason,
    //        title: 'An Error Occured'
    //    }).then(function(res) {
    //        console.log('Tapped!', res);
    //    });
    //})

    PromiseChain.then(function(data) {
        $scope.posts = Utility.formatParseArrayToFeedArray(data);
        $scope.loading = false;
    });

    $scope.doRefresh = function() {
        Posts.refresh().fetch().then(function(data) {
            $scope.posts = Utility.formatParseArrayToFeedArray(data);
            $scope.loading = false;
            $scope.$broadcast('scroll.refreshComplete');
        });
    };

    $scope.loadMore = function() {
        Posts.loadMore().then(function(data) {
            if(data.length%15||data.length<=0){
                console.log("completed loading");
                Posts.noMoreToLoad();
            }
            $scope.posts = Utility.formatParseArrayToFeedArray(data);
            $scope.loading = false;
            $scope.$broadcast('scroll.infiniteScrollComplete');
        });
    };

    $scope.moreDataCanBeLoaded = function() {
        //console.log(Posts.hasMoreToLoad());
        return Posts.hasMoreToLoad();
    };

    $scope.like = function(objectID, currentPost) {
        if(!Auth.isUserLoggedIn()) {
            $location.path('/login');
            return;
        }

        var post = Posts.get(objectID);
        console.log(post);

        currentPost.processingLike = true;
        Auth.getlikesRelation().add(post);
        Auth.saveUser().then(function() {
            currentPost.processingLike = false;
            Auth.addLike(objectID);
            post.set("likesCount", post.get('likesCount')+1);
            currentPost.likesCount = currentPost.likesCount + 1;
            return post.save();
        }, function(error) {
            currentPost.processingLike = false;
            Auth.removeLike(objectID);
        });
    };

    $scope.unlike = function(objectID, currentPost) {
        if(!Auth.isUserLoggedIn()) {
            $location.path('/login');
            return;
        }

        var post = Posts.get(objectID);

        currentPost.processingLike = true;
        Auth.getlikesRelation().remove(post);
        Auth.saveUser().then(function() {
            currentPost.processingLike = false;
            Auth.removeLike(objectID);
            post.set("likesCount", post.get('likesCount')-1);
            currentPost.likesCount = currentPost.likesCount -1;
            return post.save();
        }, function(error) {
            currentPost.processingLike = false;
            Auth.addLike(post, objectID);
        });
    };
})

.controller('CreatePostCtrl', function($scope, $ionicHistory, $location, Posts) {
    $scope.myGoBack = function() {
        $ionicHistory.goBack();
    };
    var createPost = {
        title: '',
        content: ''
    };
    $scope.createPost = createPost;
    $scope.submitting = false;

    $scope.doPost = function($invalid) {
        console.log($invalid);
        if($invalid) {
            return;
        }
        $scope.submitting = true;

        var Post = Parse.Object.extend("Posts");
        var createdPost = new Post();
        createdPost.set('title', createPost.title);
        createdPost.set('content', createPost.content);
        createdPost.set('likesCount', 0);
        createdPost.set('commentsCount', 0);
        createdPost.set('user', Parse.User.current());

        var postACL = new Parse.ACL();
        postACL.setPublicReadAccess(true);
        postACL.setWriteAccess(Parse.User.current(), true);
        createdPost.setACL(postACL);

        createdPost.save().then(function(object) {
            $scope.submitting = false;
            Posts.refresh();
            $location.path('/posts/new');
        }, function(newItem, error) {
            createPost.parseInvalidError = true;
            createPost.parseInvalidErrorMessage = error.message;
            $scope.submitting = false;
        });
    };

})

.controller('CreateCommentCtrl', function($scope, $ionicHistory, $location, $stateParams, Posts) {
    $scope.myGoBack = function() {
        $ionicHistory.goBack();
    };
    var createComment = {
        content: ''
    };

    var post = Posts.get($stateParams.post_id);
    $scope.createComment = createComment;
    $scope.submitting = false;

    $scope.doPost = function($invalid) {
        console.log($invalid);
        if($invalid) {
            return;
        }
        $scope.submitting = true;

        var Comment = Parse.Object.extend("Comments");
        var createdComment = new Comment();
        createdComment.set('content', createComment.content);
        createdComment.set('likesCount', 0);
        createdComment.set('user', Parse.User.current());
        createdComment.set('post', post);

        var postACL = new Parse.ACL();
        postACL.setPublicReadAccess(true);
        postACL.setWriteAccess(Parse.User.current(), true);
        createdComment.setACL(postACL);

        createdComment.save().then(function(object) {
            post.set('commentsCount', post.get('commentsCount')+1);
            return post.save();
        }, function(newItem, error) {
            createComment.parseInvalidError = true;
            createComment.parseInvalidErrorMessage = error.message;
            $scope.submitting = false;
        }).then(function() {
            $scope.submitting = false;
            $ionicHistory.goBack();
        });
    }

})

.controller('PostCtrl', function($scope, $stateParams, $ionicHistory, $location, Posts, Utility, Auth) {
    $scope.myGoBack = function() {
        $ionicHistory.goBack();
    };

    $scope.loading = true;
    var post = Posts.get($stateParams.post_id);
    var commentCollection = Posts.commentsForPost(post);
    var retrieveCommentsForPost = commentCollection.fetch().then(function(data) {
        $scope.comments = Utility.formatParseArrayToCommentArray(data);
        $scope.loading = false;
    });
    var PromiseChain;

    if(Auth.isUserLoggedIn()) {
        PromiseChain = Auth.retrieveParseUserCommentLikesForPost(post).then(function() {
            return retrieveCommentsForPost
        });
    } else {
        PromiseChain = retrieveCommentsForPost;
    }

    $scope.post = Utility.formatParseToFeedPost(post);

    PromiseChain.resolve();

    $scope.like = function(objectID, currentComment) {
        if(!Auth.isUserLoggedIn()) {
            $location.path('/login');
        }

        var comment = commentCollection.get(objectID);
        console.log(comment);

        currentComment.processingLike = true;
        Auth.getCommentlikesRelation().add(comment);
        Auth.saveUser().then(function() {
            currentComment.processingLike = false;
            Auth.addCommentLike(objectID);
            comment.set("likesCount", comment.get('likesCount')+1);
            currentComment.likesCount = currentComment.likesCount + 1;
            return comment.save();
        }, function(error) {
            currentComment.processingLike = false;
            Auth.removeCommentLike(objectID);
        });
    };

    $scope.unlike = function(objectID, currentComment) {
        if(!Auth.isUserLoggedIn()) {
            $location.path('/login');
        }

        var comment = commentCollection.get(objectID);

        currentComment.processingLike = true;
        Auth.getCommentlikesRelation().remove(comment);
        Auth.saveUser().then(function() {
            currentComment.processingLike = false;
            Auth.removeCommentLike(objectID);
            comment.set("likesCount", comment.get('likesCount')-1);
            currentComment.likesCount = currentComment.likesCount -1;
            return comment.save();
        }, function(error) {
            currentComment.processingLike = false;
            Auth.addCommentLike(comment, objectID);
        });
    };
})

.controller('LoginCtrl', function($scope, $location, Auth) {
    //$scope.post_type = $stateParams.post_type;
    var login = {
        noSpace: /^\s*\w*\s*$/,
        username: '',
        password: ''
    };
    $scope.login = login;
    $scope.submitting = false;

    $scope.doLogin = function($invalid) {
        if($invalid) {
            return;
        }
        $scope.submitting = true;
        Parse.User.logIn(login.username, login.password).then(function(user) {

        },
        function(error) {
            login.parseInvalidError = true;
            login.parseInvalidErrorMessage = error.message;
            $scope.submitting = false;
        }).then(function() {
            return Auth.retrieveParseUsersLikes();
        }).then(function() {
            $scope.submitting = false;
            $location.path('/posts/new');
        });
    }
})

.controller('RegisterCtrl', function($scope, $location) {
    var registration = {
        noSpace: /^\s*\w*\s*$/
    };
    $scope.registration = registration;
    $scope.submitting = false;

    $scope.doRegistration = function($invalid) {
        if($invalid) {
            return;
        }
        $scope.submitting = true;
        var user = new Parse.User();
        user.set("username", registration.username);
        user.set("password", registration.password);

        user.signUp().then(function(user) {
            $scope.submitting = false;
            $location.path('/posts/new');
        },
        function(error) {
            registration.parseInvalidError = true;
            registration.parseInvalidErrorMessage = error.message;
            $scope.submitting = false;
        });
    }

})


.controller('ChatsCtrl', function($scope, Chats) {
  $scope.chats = Chats.all();
  $scope.remove = function(chat) {
    Chats.remove(chat);
  }
})

.controller('ChatDetailCtrl', function($scope, $stateParams, Chats) {
  $scope.chat = Chats.get($stateParams.chatId);
})

.controller('AccountCtrl', function($scope) {
  $scope.settings = {
    enableFriends: true
  };
});