angular.module('starter.services', [])

.factory('Posts', function() {
    // Might use a resource here that returns a JSON array

    // Some fake testing data

    var _LIMIT = 15;
    var _filter = 'new';

    var Post = Parse.Object.extend("Posts");
    var PostsCollection = Parse.Collection.extend({
        model: Post
    });

    var newPostsCollection = Parse.Collection.extend({
        model: Post,
        query: new Parse.Query(Post).include("user").descending('createdAt').limit(_LIMIT)
    });

    var trendingPostsCollection = Parse.Collection.extend({
        model: Post,
        query: new Parse.Query(Post).include("user").greaterThan("createdAt", moment().subtract(10, 'days').toDate()).descending('likesCount').limit(_LIMIT)
    });

    var topPostsCollection = Parse.Collection.extend({
        model: Post,
        query: new Parse.Query(Post).include("user").descending('likesCount').limit(_LIMIT)
    });


    var currentParsedPosts = undefined;
    var currentPostsCollection = undefined;

    return {
        'queryLimit': _LIMIT,
        topPosts: function() {
            currentPostsCollection = new topPostsCollection();
            return currentPostsCollection;
        },
        trendingPosts: function() {
            currentPostsCollection = new trendingPostsCollection();
            return currentPostsCollection;
        },
        newPosts: function() {
            currentPostsCollection = new newPostsCollection();
            return currentPostsCollection;
        },
        refresh: function() {
            if(currentPostsCollection) {
                currentPostsCollection.query.skip(0);
                currentPostsCollection.hasMoreToLoad = undefined;
                return currentPostsCollection;
            }
        },
        loadMore: function() {
            currentPostsCollection.query.skip(currentPostsCollection.length);
            return currentPostsCollection.fetch({'add': true});
        },
        hasMoreToLoad: function() {
            //console.log(!(currentPostsCollection.hasMoreToLoad === false));
            return currentPostsCollection.hasMoreToLoad == undefined;
        },
        noMoreToLoad: function() {
            currentPostsCollection.hasMoreToLoad = false;
        },
        get: function(objectId) {
            return currentPostsCollection.get(objectId);
        },
        commentsForPost: function(post) {
            var Comment = Parse.Object.extend("Comments");
            var post_comment_query =  new Parse.Query(Comment).exists('post').equalTo("post", post).include("user").ascending('createdAt');
            var PostCommentCollection = Parse.Collection.extend({
                model: Comment,
                query: post_comment_query
            });
            return new PostCommentCollection();
        }
    };
})

.factory('Auth', function() {
    if(Parse.User.current()) {
        var likesRelation =  Parse.User.current().relation("likes");
        var CommentlikesRelation =  Parse.User.current().relation("comment_likes");
    }
    var commentLikesRelation;
    var usersLikes = []; var usersCommentLikes = [];

    var removeString = function(arr, what) {
        var found = arr.indexOf(what);

        while (found !== -1) {
            arr.splice(found, 1);
            found = arr.indexOf(what);
        }
    };

    return {
        getlikesRelation: function() {
            return Parse.User.current().relation("likes");
        },
        getCommentlikesRelation: function() {
            return Parse.User.current().relation("comment_likes");
        },
        isUserLoggedIn : function() {
            return !(Parse.User.current() == undefined)
        },
        retrieveParseUsersLikes: function() {
            return Parse.User.current().relation("likes").query().find().then(function(list) {
                angular.forEach(list, function(object) {
                    usersLikes.push(object.id);
                });
            })
        },
        getUsersLikes: function() {
            return usersLikes;
        },
        removeLike: function(id) {
            removeString(usersLikes, id);
        },
        addLike: function(id) {
            usersLikes.push(id);
        },
        saveUser: function() {
            return Parse.User.current().save();
        },
        getUsersCommentLikes: function() {
            return usersCommentLikes;
        },
        removeCommentLike: function(id) {
            removeString(usersCommentLikes, id);
        },
        addCommentLike: function(id) {
            usersCommentLikes.push(id);
        },
        retrieveParseUserCommentLikesForPost: function(post) {
            var Comment = Parse.Object.extend("Comments");
            var post_comment_query =  new Parse.Query(Comment).exists('post').equalTo("post", post).include("user").ascending('createdAt');
            var PostCommentCollection = Parse.Collection.extend({
                model: Comment,
                query: post_comment_query
            });
            var currentCommentCollection = new PostCommentCollection();
            return Parse.User.current().relation("comment_likes").query().matchesQuery("post", post).find().then(function(list) {
                usersCommentLikes = [];
                angular.forEach(list, function (object) {
                    usersCommentLikes.push(object.id);
                });
            });
        }
    }
})

.factory('Utility', function(Auth) {
   var usersLikes = [];

   return {
       formatParseToFeedPost : function(parseObject) {
           var newObject = {
               objectId: parseObject.id,
               userId: parseObject.get('user').id,
               username: parseObject.get('user').get('username'),
               commentsCount: parseObject.get('commentsCount'),
               likesCount: parseObject.get('likesCount'),
               createdAt: moment(parseObject.createdAt).fromNow(),
               modifiedAt: moment(parseObject.modifiedAt).fromNow(),
               content: parseObject.get('content'),
               title: parseObject.get('title')
           };
           newObject.liked = function() {
               return Auth.getUsersLikes().indexOf(this.objectId) > -1
           };
           newObject.totalLikesCount = function() {
               return this.liked() ? this.likesCount+1 : this.likesCount;
           };
           newObject.btn_class = function() {
               return this.liked() ? "btn-primary": "btn-default";
           };
           newObject.owned = function() {
               return (newObject.userId === Parse.User.current().id)
           };
           return newObject;
       },
       formatParseArrayToFeedArray : function(parseObjectArray) {
           var Utility = this;
           var parseResults = [];
           angular.forEach(parseObjectArray, function(v,k) {
               parseResults.push(Utility.formatParseToFeedPost(v));
           });
           return parseResults;
       },
       formatParseToFeedComment: function(parseObject) {
           var newObject = {
               objectId: parseObject.id,
               userId: parseObject.get('user').id,
               username: parseObject.get('user').get('username'),
               createdAt: moment(parseObject.createdAt).fromNow(),
               modifiedAt: moment(parseObject.modifiedAt).fromNow(),
               content: parseObject.get('content'),
               likesCount: parseObject.get('likesCount')
           };
           newObject.liked = function() {
               return Auth.getUsersCommentLikes().indexOf(this.objectId) > -1;
           };
           newObject.btn_class = function() {
               return this.liked() ? "btn-primary": "btn-default";
           };
           newObject.owned = function() {
               return (newObject.userId === Parse.User.current().id)
           };
           return newObject;
       },
       formatParseArrayToCommentArray: function(parseObjectArray) {
           var Utility = this;
           var parseResults = [];
           angular.forEach(parseObjectArray, function(v,k) {
               parseResults.push(Utility.formatParseToFeedComment(v));
           });
           return parseResults;
       }
   }
});


